package PC20060626;

import ProSW.*;
import java.util.*;

public class Alimento {
	private final String nome;
	public String getNome() { return nome; }
	
	private int unitaDiMisura;  // 0=g, 1=l, 2=u
	
	public static final int GRAMMI = 0;
	public static final int LITRI = 1;
	public static final int UNITA = 2;	
	
	public int getUnitaDiMisura() { 
		return unitaDiMisura; 
	}
	
	private double dispon;
	public double getDispon() { return dispon; }
	public void setDispon(double d) throws EccezionePrecondizioni {
		if (d < 0) throw new EccezionePrecondizioni("...");
		dispon = d;
	}
	
	private double kcal;
	public double getKCal() { return kcal; }
	public void setKCal(double k) throws EccezionePrecondizioni {
		if (k < 0) throw new EccezionePrecondizioni("...");
		kcal = k;
	}
		
	
	public Alimento(String n, int u, double d, double k, TipologiaAlimento t) 
													throws EccezionePrecondizioni {
		if (n == null) throw new EccezionePrecondizioni("...");
		nome = n;
		if (!(u == LITRI || u == GRAMMI || u == UNITA)) 
					throw new EccezionePrecondizioni("...");		
		unitaDiMisura = u;
		
		setDispon(d);
		setKCal(k);
		
		if (t == null) throw new EccezionePrecondizioni("...");
		tipologia = t;
		
	}

	// GESTIONE ASSOCIAZIONE INGREDIENTE (0..*)
	private Set<TipoLinkIngrediente> insLinkIngrediente = new HashSet<TipoLinkIngrediente>();
	
	public void inserisciLinkIngrediente(AssociazioneIngrediente pass, TipoLinkIngrediente l) throws EccezionePrecondizioni {
		if (pass == null) throw new EccezionePrecondizioni("...");
		
		insLinkIngrediente.add(l);
	}

	public void eliminaLinkIngrediente(AssociazioneIngrediente pass, TipoLinkIngrediente l) throws EccezionePrecondizioni {
		if (pass == null) throw new EccezionePrecondizioni("...");
		
		insLinkIngrediente.remove(l);
	}
		
	public Set getLinkIngrediente() {
		return (Set)((HashSet)insLinkIngrediente).clone();
	}

	// GESTIONE ASSOCIAZIONE tipologia (1..1, resp. singola, <<imm>>)
	private final TipologiaAlimento tipologia;
	public TipologiaAlimento getTipologia() { return tipologia; }


	// GESTIONE ASSOCIAZIONE incompatibileA
	/* Nota: si tratta di un'associazione a resp. doppia che incide due volte sulla stessa classe:
	         dovremmo distinguere due ruoli A e B.
	         Inoltre, si tratta di una associazione simmetrica:  
	           <A:a1, B:a2> in incompatibileA  se e solo se  <A:a2, B:a1> in incompatibileA,
	         ovvero i ruoli non sono significativi.
	         Decidiamo di realizzarla in modo "semplificato", ignorando i ruoli
	         (solo a scopo dimostrativo -- tale scelta andrebbe fatta in fase di Progetto).
	         
	         In particolare:
	         - Trattiamo l'associazione a resp. multipla come se fosse a resp. singola, ed evitiamo
	           di realizzare la classe AssociazioneIncompatibileA. 
	         - Facciamo in modo che quando il metodo a1.inserisciLink...(a2) inserisce il link
	           <a1, a2>, inserisce anche il link <a2, a1> 
	           (ovvero a2.inserisciLink...(a1)). In questo modo realizziamo entrambi i "versi" dell'associazione (grazie alla simmetria).
	         - Si noti che, proprio perche' l'associazione incide su una sola classe,
	           il cliente non ha alcun modo di inserire i link in modo asimmetrico: non abbiamo
	           bisogno di alcun pass.
	*/
	
	// Un insieme di alimenti, come se l'assoc fosse a resp. singola...
	private Set<Alimento> insLinkIncompatibileA = new HashSet<Alimento>();
	
	
	public void inserisciLinkIncompatibileA(Alimento a) throws EccezionePrecondizioni {
		if (a == null) throw new EccezionePrecondizioni("...");
		
		// Inserimento "simmetrico", come se l'assoc fosse a resp. doppia
		a.insLinkIncompatibileA.add(this);
		this.insLinkIncompatibileA.add(a);
		
		/* Di fatto stiamo facendo la stessa cosa di un ipotetico metodo
		   static AssociazioneIncompatibileA.inserisciLink...().  L'eliminazione
		   della classe AssociazioneIncomp...  possibile solo perche' entrambi i terminali
		   dell'associazione sono la stessa classe (il cliente non puo' inserire link
		   in modo asimmetrico in alcun modo!).
		   
		   La simmetria dell'associazione ci garantisce di non perdere informazione.
		 */
	}

	public void eliminaLinkIncompatibileA(Alimento a) throws EccezionePrecondizioni {
		if (a == null) throw new EccezionePrecondizioni("...");
		
		this.insLinkIncompatibileA.remove(a);
		a.insLinkIncompatibileA.remove(this);
	}

	public Set<Alimento> getAlimentiIncompatibili() {
		/* Sono tutti gli alimenti incompatibili con this.
		   Il metodo inserisci ha gestito correttamente l'inserimento di un link <a, this>,
		   inserendo anche <this, a>, che sara' quindi presente nell'insieme this.insLinkIncomp...
		*/
		return (Set<Alimento>)((HashSet<Alimento>)insLinkIncompatibileA).clone();
	}

	public boolean compatibileCon(Alimento altro) throws EccezionePrecondizioni { 
		if (altro == null) throw new EccezionePrecondizioni("...");
		
		if (this.insLinkIncompatibileA.contains(altro)) return false;
			
		// Non sono esplicitamente incompatibili, ora guardo se sono di tipologie incompabili
		if (this.tipologia.compatibileCon(altro.tipologia)) return true;
		return false;
	}
	
	
} //:~
